from geant4_pybind import *
#from MuNuclearBuilder import MuNuclearBuilder

def list_available_physics():
    import geant4_pybind as g4
    for v in sorted(dir(g4)):
      if ("Physics" in v) and ("G4V" not in v) and ("Vector" not in v):
        print(v)
    print(" ")

class CustomPhysicsList(G4VModularPhysicsList):

  def __init__(self, mode='all_phys', optics=True):
    super().__init__()

    self.phys_constructors = {}
    
    if mode == 'em_cascade':
      self.phys_constructors["decay"] = G4DecayPhysics()
      self.phys_constructors["em"] = G4EmStandardPhysics_option4()   # G4EmStandardPhysics(),  G4EmStandardPhysics_option1()
    elif mode == 'all_phys':
      self.phys_constructors["decay"] = G4DecayPhysics()
      self.phys_constructors["em"] = G4EmStandardPhysics_option4()
      self.phys_constructors["em_extra"] = G4EmExtraPhysics()  # for muon nuclear interaction
      self.phys_constructors["rad_decay"] = G4RadioactiveDecayPhysics()
      self.phys_constructors["hadron"] = G4HadronPhysicsFTFP_BERT()
      #self.phys_constructors["stopping"] = G4StoppingPhysics()
    elif mode == 'fast':
      self.phys_constructors["decay"] = G4DecayPhysics()
      self.phys_constructors["em"] = G4EmStandardPhysics_option1()
      self.phys_constructors["em_extra"] = G4EmExtraPhysics()  # for muon nuclear interaction
      self.phys_constructors["hadron"] = G4HadronPhysicsFTFP_BERT()
      
    self.phys_constructors["stopping"] = G4StoppingPhysicsFritiofWithBinaryCascade()
    
    if optics:
      self.phys_constructors["optical"] = G4OpticalPhysics()

    for key, phys in self.phys_constructors.items():
      self.RegisterPhysics(phys)
      phys.SetVerboseLevel(0)

    #list_available_physics()




from g4camp.g4camp import g4camp

sim = g4camp(primary_generator='gps', optics=False)
sim.setDetectorHeight(1000)
sim.setDetectorRadius(500)
sim.setSkipMinMax(0.0, 0.01)
sim.setVerbose(tracking_verbose=1)
sim.configure()

# One event at 0 0 0
sim.applyGeant4Command('/gps/particle e-')
sim.applyGeant4Command('/gps/energy 1 TeV')
sim.applyGeant4Command('/gps/position 0 0 0')
sim.applyGeant4Command('/gps/direction 0 0 1')
#
for data in sim.run(n_events=1):
    particles = data.particles
    tracks = data.tracks
    print(f"\n{len(tracks):>6} tracks, {len(particles):>6} cascade starters;\n")
    
# Many events at random positions and directions
sim.applyGeant4Command('/gps/pos/type Volume')
sim.applyGeant4Command('/gps/pos/shape Cylinder')
sim.applyGeant4Command('/gps/pos/halfz 400 m')
sim.applyGeant4Command('/gps/pos/radius 400 m')
sim.applyGeant4Command('/gps/pos/center 0 0 500 m')
sim.applyGeant4Command('/gps/dir/type iso')
sim.configure()
sim.setVerbose(tracking_verbose=0)
#
for ievt, data in enumerate(sim.run(n_events=10)):
    particles = data.particles
    tracks = data.tracks
    primary = tracks[0]
    uid, gen, parent_uid, pdgid, x, y, z, t, E, l = primary
    print(f"Event #{ievt}")
    print(f"Position:   {x:.1f}, {y:.1f}, {z:.1f} m")
    print(f"{len(tracks):>6} tracks, {len(particles):>6} cascade starters;\n")



from geant4_pybind import *
from g4camp.g4camp import g4camp
import sys

n_events = 10
gun_args = { 'particle':   'mu-', 
             'energy_GeV': 100000,
             'position_m': [0,0,10],
             'direction':  [0,0,1]  }

app = g4camp(optics=False, primary_generator='gun', gun_args=gun_args)
ui = G4UIExecutive(len(sys.argv), sys.argv)
visManager = G4VisExecutive("Quiet")
visManager.Initialize()
app.configure() # should be after initialization of visual manager
app.applyGeant4Command("/control/execute",  ["vis.mac"])
ui.SessionStart()
